/*
	Publimark
 
	Copyright (c) 2010, Gatan Le Guelvouit <contact@gleguelv.org>
 
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.
 
	This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more details.
 
	You should have received a copy of the GNU General Public
	License along with this program; if not, write to the Free
	Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
	MA 02111-1307, USA.
*/

#include "crypto.h"

using namespace Botan;

// Create a pair of keys 
void crypto::get_pair(string& pub, string& prv)
{
	AutoSeeded_RNG rng;
	
	RSA_PrivateKey k(rng, CRYPTO_BITS);
	
	pub=X509::PEM_encode(k);
	prv=PKCS8::PEM_encode(k, rng, PASSPH);
}

// Encrypt a string using public key file 
string crypto::encrypt(const string& s, const char* file_pub)
{
	// I must check myself if the file exists
	{
		std::ifstream tmp(file_pub);
	
		if(!tmp) ERR("Cannot open key file " << file_pub); 
		
		tmp.close();
	}
		
	DataSource_Stream key_file(file_pub);
	AutoSeeded_RNG rng;
	
	X509_PublicKey*	Kpub 	= X509::load_key(key_file); 
	RSA_PublicKey*	rsakey 	= dynamic_cast<RSA_PublicKey*>(Kpub);
	PK_Encryptor*  	enc 	= get_pk_encryptor(*rsakey, "EME1(SHA-1)");
	
	byte buffer[s.size()];
	for(unsigned int i=0; i<s.size(); i++) buffer[i]=(byte)(int(s[i]) + 128);
	
	SecureVector<byte> out = enc->encrypt(buffer, s.size(), rng);
	
	LOG("--  [crypto::encrypt] Rate : " << out.size()/s.size() << endl);
	
	string ret;
	for(unsigned int i=0; i<out.size(); i++) ret += (char)(int(out[i]) - 128);
	
	return ret;
}

// Decrypt a buffer using private key file 
string crypto::decrypt(const string& s, const char* file_prv)
{
	// I must check myself if the file exists
	{
		std::ifstream tmp(file_prv);
	
		if(!tmp) ERR("Cannot open key file " << file_prv);
		
		tmp.close();
	}
	
	DataSource_Stream key_file(file_prv);
	AutoSeeded_RNG rng;
	
	PKCS8_PrivateKey* 	Kprv 	= (PKCS8::load_key(key_file, rng, PASSPH)); 
	RSA_PrivateKey* 	rsakey 	= dynamic_cast<RSA_PrivateKey*>(Kprv);
	PK_Decryptor* 		dec 	= get_pk_decryptor(*rsakey, "EME1(SHA-1)"); 
	
	byte buffer[s.size()];
	for(unsigned int i=0; i<s.size(); i++) buffer[i]=(byte)(int(s[i]) + 128);
	
	try
	{
		SecureVector<byte> out = dec->decrypt(buffer, s.size()); 
		
		string ret;
		for(unsigned int i=0; i<out.size(); i++) ret+=(char)(int(out[i]) - 128);
		
		return ret;
	
	} 
	catch(std::exception& e)
	{
		ERR("Source file does not contain stego-data");
	}
}
