/*
	Publimark
 
	Copyright (c) 2010, Gaëtan Le Guelvouit <contact@gleguelv.org>
 
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.
 
	This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more details.
 
	You should have received a copy of the GNU General Public
	License along with this program; if not, write to the Free
	Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
	MA 02111-1307, USA.
*/


#include "mtrand.h"

#define N 624
#define M 397

#define MIX(u,v) 	(((u) & 0x80000000UL) | ((v) & 0x7fffffffUL))
#define SWAP(u,v) 	((MIX(u,v) >> 1) ^ ((v)&1UL ? 0x9908b0dfUL : 0UL))


static unsigned long state[N]; // Vecteur d'état 
static unsigned long *next;

static int left  = 1;
static bool init = false;


void mtsrand(unsigned long s)
{
    state[0] = s & 0xffffffffUL;
    
	for(int j=1; j<N; j++) 
	{
        state[j] =  (1812433253UL*(state[j-1]^(state[j-1]>>30)) + j); 
      	state[j] &= 0xffffffffUL; 
    }

    left = 1; 
	init = true;
}

static void next_state(void)
{
    unsigned long *p = state;

    if(!init) mtsrand(9123UL); // Graine par défaut 

    left = N;
    next = state;
    
    for(int i=N-M+1; --i; p++) *p = p[M]^SWAP(p[0], p[1]);

    for(int i=M; --i; p++) *p = p[M-N]^SWAP(p[0], p[1]);

    *p = p[M-N]^SWAP(p[0], state[0]);
}

unsigned long mtrand()
{
    unsigned long y;

    if(--left==0) next_state();
    y = *next++;

    y ^= (y >> 11);
    y ^= (y << 7) & 0x9d2c5680UL;
    y ^= (y << 15) & 0xefc60000UL;
    y ^= (y >> 18);

    return y;
}


